@@ -8,10 +8,10 @@ using namespace Eigen;
 //[[Rcpp::depends(RcppEigen)]]
 //[[Rcpp::plugins(openmp)]
 
-void coglasso_sub(Eigen::MatrixXd& S, Eigen::MatrixXd& W, Eigen::MatrixXd& T, int pX, int p, double ialpha, Eigen::MatrixXd& Lambda_star, int& df, int& converged, bool scr, unsigned int index);
+void coglasso_sub(Eigen::MatrixXd& S, Eigen::MatrixXd& W, Eigen::MatrixXd& T, int pX, int p, double ialpha, double ic, Eigen::MatrixXd& Lambda_star, int& df, int& converged, bool scr, unsigned int index);
 
 //[[Rcpp::export]]
-List co_glasso(Eigen::Map<Eigen::MatrixXd> S, int pX, Eigen::Map<Eigen::MatrixXd> hpars, bool scr, bool verbose, bool cov_output)
+List co_glasso(Eigen::Map<Eigen::MatrixXd> S, int pX, Eigen::Map<Eigen::MatrixXd> hpars, Eigen::Map<Eigen::MatrixXd> T_guess, bool scr, bool verbose, bool cov_output)
 {
   // Caller function, initiates what must be initiated and calls the main algorithm for 
   // every combination of hyperparameters.
@@ -58,6 +58,7 @@ List co_glasso(Eigen::Map<Eigen::MatrixXd> S, int pX, Eigen::Map<Eigen::MatrixXd
         double alpha = hpars(i, 0);
         double lambda_w = hpars(i, 1);
         double lambda_b = hpars(i, 2);
+        double c = hpars(i, 3);
 
         // Matrix of lambda_w and lambda_b multiplied by alpha is built here
         MatrixXd Lambda_star_ij(p, p);
@@ -86,6 +87,7 @@ List co_glasso(Eigen::Map<Eigen::MatrixXd> S, int pX, Eigen::Map<Eigen::MatrixXd
                 if (break_flag > 1) break;
                 // should I use alpha*S (or maybe just lambda) since I'll be subtracting from alpha*S?
                 if (alpha * S(row_i, col_i) > Lambda_star_ij(row_i, col_i) or alpha * S(row_i, col_i) < -Lambda_star_ij(row_i, col_i)) break_flag++;
+                // if (S(row_i, col_i) > Lambda_star_ij(row_i, col_i) or S(row_i, col_i) < -Lambda_star_ij(row_i, col_i)) break_flag++;
             }
             if (break_flag > 1) z.push_back(row_i);
         }
@@ -125,8 +127,8 @@ List co_glasso(Eigen::Map<Eigen::MatrixXd> S, int pX, Eigen::Map<Eigen::MatrixXd
                 if (zero_sol) {
                     // maybe initialize W to alpha * S?
                     sub_W(ii, jj) = S(ii, jj);
-                    sub_T(ii, jj) = ii == jj ? 1 : 0;
-                }
+                    sub_T(ii, jj) = ii == jj ? 1 : T_guess(ii, jj);
+                  }
                 else {
                     sub_W(ii, jj) = (*(tmp_cov_p.back()))(ii, jj);
                     sub_T(ii, jj) = (*(tmp_icov_p.back()))(ii, jj);
@@ -134,7 +136,6 @@ List co_glasso(Eigen::Map<Eigen::MatrixXd> S, int pX, Eigen::Map<Eigen::MatrixXd
             }
         }
 
-
         // Anti explosion warm start
         // for (int ii = 0; ii < p; ii++) {
         //     for (int jj = 0; jj < p; jj++) {
@@ -169,12 +170,12 @@ List co_glasso(Eigen::Map<Eigen::MatrixXd> S, int pX, Eigen::Map<Eigen::MatrixXd
         {
             if (verbose) {
                 if (scr)
-                    Rcout << "\rConducting the collaborative graphical lasso (coglasso) wtih lossy screening....in progress: " << floor(100 * (1. - (1. * i / nhpars))) << "%";
+                    Rcout << "Conducting the collaborative graphical lasso (coglasso) wtih lossy screening....in progress: " << floor(100 * (1. - (1. * i / nhpars))) << "%\r";
                 if (!scr)
-                    Rcout << "\rConducting the collaborative graphical lasso (coglasso) wtih lossless screening....in progress: " << floor(100 * (1. - (1. * i / nhpars))) << "%";
+                    Rcout << "Conducting the collaborative graphical lasso (coglasso)....in progress: " << floor(100 * (1. - (1. * i / nhpars))) << "%\r";
             }
 
-            coglasso_sub(sub_S, sub_W, sub_T, pX, p, alpha, Lambda_star_ij, sub_df, converged, scr, i);
+            coglasso_sub(sub_S, sub_W, sub_T, pX, p, alpha, c, Lambda_star_ij, sub_df, converged, scr, i);
             zero_sol = false;
         }
 
@@ -240,6 +241,14 @@ List co_glasso(Eigen::Map<Eigen::MatrixXd> S, int pX, Eigen::Map<Eigen::MatrixXd
         }
 
     }
+    
+    if (verbose) {
+      if (scr)
+        Rcout << "Conducting the collaborative graphical lasso (coglasso) wtih lossy screening....done            \n";
+      if (!scr)
+        Rcout << "Conducting the collaborative graphical lasso (coglasso)....done            \n";
+    }
+    
     List path, icov, cov;
     for (unsigned int i = 0; i < nhpars; i++) {
         path.push_back(*(tmp_path_p[nhpars - 1 - i]));
@@ -284,7 +293,7 @@ List co_glasso(Eigen::Map<Eigen::MatrixXd> S, int pX, Eigen::Map<Eigen::MatrixXd
 //      used in the current coglasso implementation;
 // index is the index of the current combination of hyperparameters (that goes from number of combinations to 0).
 //
-void coglasso_sub(Eigen::MatrixXd& S, Eigen::MatrixXd& W, Eigen::MatrixXd& T, int pX, int p, double ialpha, Eigen::MatrixXd& Lambda_star, int& df, int& converged, bool scr, unsigned int index)
+void coglasso_sub(Eigen::MatrixXd& S, Eigen::MatrixXd& W, Eigen::MatrixXd& T, int pX, int p, double ialpha, double ic, Eigen::MatrixXd& Lambda_star, int& df, int& converged, bool scr, unsigned int index)
 {
     int i, j, k; //initialize indices
     int rss_idx, w_idx;
@@ -429,7 +438,8 @@ void coglasso_sub(Eigen::MatrixXd& S, Eigen::MatrixXd& W, Eigen::MatrixXd& T, in
                                     r = r - W(j, rss_idx) * T(rss_idx, i);
                                 }
                                 else {
-                                    r = r + (1 - ialpha) * W(j, rss_idx) * T(rss_idx, i);
+                                    // r = r + (1 - ialpha) * W(j, rss_idx) * T(rss_idx, i);
+                                    r = r - ialpha * (1 - ic) * W(j, rss_idx) * T(rss_idx, i);
                                 }
                                 
                             }
@@ -439,7 +449,7 @@ void coglasso_sub(Eigen::MatrixXd& S, Eigen::MatrixXd& W, Eigen::MatrixXd& T, in
                             {
                                 rss_idx = idx_a(k, i);
                                 if (rss_idx < pX) {
-                                    r = r + (1 - ialpha) * W(j, rss_idx) * T(rss_idx, i);
+                                    r = r - ialpha * (1 - ic) * W(j, rss_idx) * T(rss_idx, i);
                                 }
                                 else {
                                     r = r - W(j, rss_idx) * T(rss_idx, i);
@@ -527,7 +537,8 @@ void coglasso_sub(Eigen::MatrixXd& S, Eigen::MatrixXd& W, Eigen::MatrixXd& T, in
                                         r = r - W(w_idx, rss_idx) * T(rss_idx, i);
                                     }
                                     else {
-                                        r = r + (1 - ialpha) * W(w_idx, rss_idx) * T(rss_idx, i);
+                                        // r = r + (1 - ialpha) * W(w_idx, rss_idx) * T(rss_idx, i);
+                                        r = r - ialpha * (1 - ic) * W(w_idx, rss_idx) * T(rss_idx, i);
                                     }
                                 }
                             }
@@ -537,7 +548,8 @@ void coglasso_sub(Eigen::MatrixXd& S, Eigen::MatrixXd& W, Eigen::MatrixXd& T, in
                                 {
                                     rss_idx = idx_a(k, i);
                                     if (rss_idx < pX) {
-                                        r = r + (1 - ialpha) * W(w_idx, rss_idx) * T(rss_idx, i);
+                                        // r = r + (1 - ialpha) * W(w_idx, rss_idx) * T(rss_idx, i);
+                                        r = r - ialpha * (1 - ic) * W(w_idx, rss_idx) * T(rss_idx, i);
                                     }
                                     else {
                                         r = r - W(w_idx, rss_idx) * T(rss_idx, i);
